package pl.edu.pwr.tkubik.simplejpa;

import java.sql.SQLException;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.annotation.Bean;
import org.h2.tools.Server;

import pl.edu.pwr.tkubik.simplejpa.model.Address;
import pl.edu.pwr.tkubik.simplejpa.model.Owner;
import pl.edu.pwr.tkubik.simplejpa.model.Pet;
import pl.edu.pwr.tkubik.simplejpa.model.PetType;
import pl.edu.pwr.tkubik.simplejpa.service.AddressService;
import pl.edu.pwr.tkubik.simplejpa.service.OwnerService;
import pl.edu.pwr.tkubik.simplejpa.service.PetService;



@SpringBootApplication
public class SimpleJpaApplication implements CommandLineRunner {
	private static final Logger log = LoggerFactory.getLogger(SimpleJpaApplication.class);
    private static ConfigurableApplicationContext ctx;
	private final PetService petService;
	private final OwnerService ownerService;
	private final AddressService addressService;

	//private static boolean end = false;

	@Autowired
	public SimpleJpaApplication(PetService petService, OwnerService ownerService, AddressService addressService) {
		this.petService = petService;
		this.ownerService = ownerService;
		this.addressService = addressService;
	}

	@Bean(initMethod = "start", destroyMethod = "stop")
	public Server h2Server() throws SQLException {
		return Server.createTcpServer("-tcp", "-tcpAllowOthers", "-tcpPort", "9092");
	}

	public static void main(String[] args) {
		ctx = SpringApplication.run(SimpleJpaApplication.class, args);
		// while(!end) Thread.currentThread().yield();
		// System.exit(SpringApplication.exit(ctx));
	}

	@Override
	public void run(String... args) throws Exception {
		org.h2.tools.Server server = org.h2.tools.Server.createTcpServer().start();
		Address a = new Address()
				.withStreetAddress("Wyspianskiego 11")
				.withZipCode("50-400")
				.withCity("Wroclaw");
		a = addressService.addAddress(a);
		Owner o1 = new Owner(0, "John", "555000001", a);
		Owner o2 = new Owner(0, "Adam", "555000002", a);
        List<Address> la = addressService.findAddressByCity("Wroclaw");
		log.info(la.toString());
		o1 = ownerService.addOwner(o1);
		o2 = ownerService.addOwner(o2);

		Pet p1 = new Pet("Tom", PetType.CAT, o1);
		p1 = petService.addPet(p1);
		Pet p2 = new Pet("Jerry", PetType.MOUSE, o1);
		p2 = petService.addPet(p2);

		for (Pet pet : petService.findAllPets()) {
			log.info(pet.toString());
		}
		log.info("THIS IS THE END");
		//end = true;
	}
/*

1. Modelling many to many relations
https://www.baeldung.com/jpa-many-to-many

2. Spring Boot with H2 Database
https://www.baeldung.com/spring-boot-h2-database

3. Accessing H2 Database running within SpringBootApplication

How to enable H2 Database Server Mode in Spring Boot
https://stackoverflow.com/questions/55830010/how-to-enable-h2-database-server-mode-in-spring-boot

How to access in memory h2 database of one spring boot application from another spring boot application
https://stackoverflow.com/questions/43256295/how-to-access-in-memory-h2-database-of-one-spring-boot-application-from-another/43276769#43276769

4. Just to inform
### Not needed because of existing dependencies in pom.xml
spring.datasource.driverClassName=org.h2.Driver
spring.jpa.database-platform=org.hibernate.dialect.H2Dialect

5. Browsing H2 in mem database
[http://localhost:8080/h2-console]

#### application.properties
spring.datasource.url=jdbc:h2:mem:testdb
spring.jpa.show-sql = true
spring.jpa.hibernate.ddl-auto=create
spring.datasource.username=sa
spring.datasource.password=password
spring.h2.console.enabled=true
spring.h2.console.path=/h2-console
spring.h2.console.settings.trace=true
spring.h2.console.settings.web-allow-others=true

#### pom.xml (needed to connect by h2-console)

        <dependency>
			<groupId>org.springframework.boot</groupId>
			<artifactId>spring-boot-starter-web</artifactId>
		</dependency>

#### in source code of SpringBootApplication (needed to connect by dbeaver)

    @Bean(initMethod = "start", destroyMethod = "stop")
    public Server h2Server() throws SQLException {
        return Server.createTcpServer("-tcp", "-tcpAllowOthers", "-tcpPort", "9092");
    }

#### Settings for h2-console
#### running on http://localhost:8080/h2-console/
Driver class: org.h2.Driver
JDBC URL: jdbc:h2:mem:testdb

##### Settings for dbeaver
URI template: jdbc:h2:tcp://localhost:9092/mem:testdb
Default database: mem.testdb


##### changes if database is going to be stored in a file
##### in application.properties:
spring.datasource.url=jdbc:h2:./testdb
##### in dbeaver settings
JDBC URL: jdbc:h2:tcp://localhost:9092/./testdb

6. Lifecycle
https://thorben-janssen.com/persist-save-merge-saveorupdate-whats-difference-one-use/

7. Why Set Is Better Than List in @ManyToMany
https://dzone.com/articles/why-set-is-better-than-list-in-manytomany

*/
}
